/* A variant of `nautyex3.c' keeping cut-loose groups
   and printing them again at the end of the session.

   The program also reports the addresses of the coset
   representative arrays (levelinfo[0].replist).

   19-Dec-18, Anton Voropaev (anton.n.voropaev@gmail.com)
*/

/* This program prints the entire automorphism group of an n-vertex
   polygon, where n is a number supplied by the user. 
*/

#include "nauty.h"    /* which includes <stdio.h> */
#include "naugroup.h"

/**************************************************************************/

void
writeautom(int *p, int n)
/* Called by allgroup.  Just writes the permutation p. */
{
    int i;

    for (i = 0; i < n; ++i) printf(" %2d",p[i]); printf("\n");
}

void
writegroup(grouprec *g, int n)
{
    printf("Automorphisms of C[%d] (replist = %p):\n",n,g->levelinfo[0].replist);
    allgroup(g,writeautom);
}

/**************************************************************************/

int
main(int argc, char *argv[])
{
    DYNALLSTAT(graph,g,g_sz);
    DYNALLSTAT(int,lab,lab_sz);
    DYNALLSTAT(int,ptn,ptn_sz);
    DYNALLSTAT(int,orbits,orbits_sz);
    DYNALLSTAT(grouprec*,grp,grp_sz);
    static DEFAULTOPTIONS_GRAPH(options);
    statsblk stats;

    int c,n,m,v;
    grouprec *group;

 /* The following cause nauty to call two procedures which
        store the group information as nauty runs. */
        
    options.userautomproc = groupautomproc;
    options.userlevelproc = grouplevelproc;

    while (1)
    {
        printf("\nenter n and 0/1 (cut loose) : ");
        if (scanf("%d %d",&n,&c) == 2 && n > 0)
        {
            m = SETWORDSNEEDED(n);
            nauty_check(WORDSIZE,m,n,NAUTYVERSIONID);

            DYNALLOC2(graph,g,g_sz,m,n,"malloc");
            DYNALLOC1(int,lab,lab_sz,n,"malloc");
            DYNALLOC1(int,ptn,ptn_sz,n,"malloc");
            DYNALLOC1(int,orbits,orbits_sz,n,"malloc");

            EMPTYGRAPH(g,m,n);
            for (v = 0; v < n; ++v) ADDONEEDGE(g,v,(v+1)%n,m);

            densenauty(g,lab,ptn,orbits,&options,&stats,m,n,NULL);

            if(c)
            {
                m = grp_sz;
                DYNALLOC1(grouprec*,grp,grp_sz,n+1,"malloc");
                while (m < grp_sz) grp[m++] = NULL;

                group = grp[n] = groupptr(TRUE);
            }
            else

         /* Get a pointer to the structure in which the group information
            has been stored.  If you use TRUE as an argument, the
            structure will be "cut loose" so that it won't be used
            again the next time nauty() is called.  Otherwise, as
            here, the same structure is used repeatedly. */
                
                group = groupptr(FALSE);

         /* Expand the group structure to include a full set of coset
            representatives at every level.  This step is necessary
            if allgroup() is to be called. */
                
            makecosetreps(group);

         /* Call the procedure writeautom() for every element of the group.
            The first call is always for the identity. */
                
            writegroup(group,n);
        }
        else
            break;
    }

    puts("\nCut-loose groups once again\n");

    for (n = 1; n < grp_sz; ++n)
        if (grp[n])
            writegroup(grp[n],n);
    exit(0);
}
