/* A `cut loose' variant of `nautyex3.c' calling `freegroup' and `free'.

   The program does not provide the original input and output.
   It is intended to perform many identical iterations and report
   the initial and the final working set sizes. The implementation
   is platform-specific (lines 31-33).

   Command-line arguments:
   number of vertices   (small, 10);
   number of iterations (large, 1000000);
   mode (0-2):
       do not call `makecosetreps',
       call `makecosetreps',
       call `groupptr' after `makecosetreps'.

   19-Dec-18, Anton Voropaev (anton.n.voropaev@gmail.com)
*/

/* This program prints the entire automorphism group of an n-vertex
   polygon, where n is a number supplied by the user. 
*/

#include "nauty.h"    /* which includes <stdio.h> */
#include "naugroup.h"

/**************************************************************************/

void
writestat(void)
{
    system("grep -i vmsize /proc/$("
           "grep -i ppid /proc/$$/status | sed 's/[^0-9]//g'"
           ")/status");
}

/**************************************************************************/

int
main(int argc, char *argv[])
{
    DYNALLSTAT(graph,g,g_sz);
    DYNALLSTAT(int,lab,lab_sz);
    DYNALLSTAT(int,ptn,ptn_sz);
    DYNALLSTAT(int,orbits,orbits_sz);
    static DEFAULTOPTIONS_GRAPH(options);
    statsblk stats;

    int c,i,n,m,v;
    grouprec *group;

 /* The following cause nauty to call two procedures which
        store the group information as nauty runs. */
        
    options.userautomproc = groupautomproc;
    options.userlevelproc = grouplevelproc;

    n = atoi(argv[1]);
    i = atoi(argv[2]);
    c = atoi(argv[3]);

    writestat();

    while (i--)
    {
            m = SETWORDSNEEDED(n);
            nauty_check(WORDSIZE,m,n,NAUTYVERSIONID);

            DYNALLOC2(graph,g,g_sz,m,n,"malloc");
            DYNALLOC1(int,lab,lab_sz,n,"malloc");
            DYNALLOC1(int,ptn,ptn_sz,n,"malloc");
            DYNALLOC1(int,orbits,orbits_sz,n,"malloc");

            EMPTYGRAPH(g,m,n);
            for (v = 0; v < n; ++v) ADDONEEDGE(g,v,(v+1)%n,m);

            densenauty(g,lab,ptn,orbits,&options,&stats,m,n,NULL);

         /* Get a pointer to the structure in which the group information
            has been stored.  If you use TRUE as an argument, the
            structure will be "cut loose" so that it won't be used
            again the next time nauty() is called.  Otherwise, as
            here, the same structure is used repeatedly. */
                
            group = groupptr(TRUE);

         /* Expand the group structure to include a full set of coset
            representatives at every level.  This step is necessary
            if allgroup() is to be called. */
                
            if (c != 0) makecosetreps(group);

            if (c == 2) groupptr(TRUE);

         /* Call the procedure writeautom() for every element of the group.
            The first call is always for the identity. */
                
            freegroup(group);
            free(group);
    }
    writestat();
    exit(0);
}
